#include "UnicodeUtils.h"

#include "Win32BaseIncludes.h"

#include <vector>
#include <stdexcept>

std::string UnicodeUtils::utf16ToUtf8String(const std::wstring& input)
{
    if (input.empty())
    {
        return std::string();
    }

#ifdef _WIN32
    const auto size = ::WideCharToMultiByte(CP_UTF8, 0, &input[0], static_cast<int>(input.size()), nullptr, 0, nullptr, nullptr);

    std::string result(size, 0);
    ::WideCharToMultiByte(CP_UTF8, 0, &input[0], static_cast<int>(input.size()), &result[0], size, nullptr, nullptr);
    return result;
#else
    throw std::logic_error("Not implemented");
#endif
}

std::wstring UnicodeUtils::utf8ToUtf16WString(const std::string& input)
{
    if (input.empty())
    {
        return std::wstring();
    }

#ifdef _WIN32
    const auto charsNeeded = ::MultiByteToWideChar(CP_UTF8, 0, input.data(), static_cast<int>(input.size()), nullptr, 0);
    std::vector<wchar_t> buffer(charsNeeded);
    const auto charsConverted = ::MultiByteToWideChar(CP_UTF8, 0, input.data(), static_cast<int>(input.size()), &buffer[0], static_cast<int>(buffer.size()));
    return std::wstring(&buffer[0], charsConverted);
#else
    throw std::logic_error("Not implemented");
#endif
}
