#include "HttpRequest.h"

#include "StringUtils.h"

#include <sstream>

HttpRequest::HttpRequest(Verb verb, const std::string& path)
    :   mVerb(verb),
        mPath(path)
{

}

HttpRequest::Verb HttpRequest::getVerb() const
{
    return mVerb;
}

std::string HttpRequest::getPath() const
{
    return mPath;
}

void HttpRequest::parseMessage(const std::string& message)
{
    std::stringstream ss(message);

    std::string buffer;
    bool firstLine {true};

    std::vector<std::string> headers;
    while(std::getline(ss, buffer, '\n'))
    {
        if (firstLine)
        {
            parseFirstLine(buffer);
            firstLine = false;
        }
        else
        {
            headers.push_back(buffer);
        }
    }
    mHeader.parse(headers);
}

void HttpRequest::parseFirstLine(const std::string& line)
{
    bool inPath{false};
    bool inMethod{true};
    bool inProtocol{false};

    for (std::size_t idx=0; idx<line.size();idx++)
    {
        const auto c = line[idx];
        if (inPath)
        {
            if (StringUtils::isSpace(c))
            {
                inPath = false;
                inMethod = true;
            }
            else
            {
                mMethod.push_back(c);
            }
        }
        else if (inMethod)
        {
            if (StringUtils::isSpace(c))
            {
                inMethod = false;
                inProtocol = true;
            }
            else
            {
                mPath.push_back(c);
            }
        }
        else if (inProtocol)
        {
            mProtocolVersion.push_back(c);
        }
    }
}
