#include "NetworkManager.h"
#ifdef __linux__
#include "UnixSocketInterface.h"
#else
#include "Win32WebServer.h"
#endif

#include <iostream>

NetworkManager::NetworkManager()
    : mActiveSockets(),
      mSocketInterface()
{

}

NetworkManager::~NetworkManager()
{

}

std::unique_ptr<NetworkManager> NetworkManager::Create()
{
    return std::make_unique<NetworkManager>();
}

void NetworkManager::initialize()
{
#ifdef __linux__
    mSocketInterface = UnixSocketInterface::Create();
#endif
}

void NetworkManager::runHttpServer(AbstractWebApp* webApp)
{
#ifdef _WIN32
    Win32WebServer server(webApp);
    server.initialize();
    server.run();
#else
    if (!mSocketInterface)
    {
        initialize();
    }

    auto socket = Socket::Create();
    mSocketInterface->InitializeSocket(socket);
    mSocketInterface->Listen(socket);
    mSocketInterface->Run(socket);
#endif
}

void NetworkManager::runHttpClient()
{
    if (!mSocketInterface)
    {
        initialize();
    }

    if (!mSocketInterface)
    {
        return;
    }

    auto socket = Socket::Create();
    mSocketInterface->InitializeSocket(socket, "127.0.0.1");
    mSocketInterface->Write(socket, "Hello Friend");
}

void NetworkManager::shutDown()
{

}
