#include "RectangleNode.h"

#include "Rectangle.h"

#include "MeshPrimitives.h"
#include "SceneInfo.h"

RectangleNode::RectangleNode(const Point& loc, double width, double height)
    : GeometryNode(loc),
      mWidth(width),
      mHeight(height)
{

}

std::unique_ptr<RectangleNode> RectangleNode::Create(const Point& loc, double width, double height)
{
    return std::make_unique<RectangleNode>(loc, width, height);
}

GeometryNode::Type RectangleNode::getType()
{
    return GeometryNode::Type::Rectangle;
}

double RectangleNode::getWidth() const
{
    return mWidth;
}

double RectangleNode::getHeight() const
{
    return mHeight;
}

void RectangleNode::setWidth(double width)
{
    if (mWidth != width)
    {
        mTransformIsDirty = true;
        mWidth = width;
    }
}

void RectangleNode::setHeight(double height)
{
    if (mHeight != height)
    {
        mTransformIsDirty = true;
        mHeight = height;
    }
}

void RectangleNode::createOrUpdateGeometry(SceneInfo* sceneInfo)
{
    if (sceneInfo->mSupportsGeometryPrimitives)
    {
        auto rect = std::make_unique<ntk::Rectangle>(Point{ 0, 0 }, 1, 1);
        mBackgroundItem = std::make_unique<SceneModel>(std::move(rect));
    }
    else
    {
        auto mesh = MeshPrimitives::buildRectangleAsTriMesh();
        mBackgroundItem = std::make_unique<SceneModel>(std::move(mesh));
        
    }
    mBackgroundItem->setName(mName + "_Model");
}

void RectangleNode::updateTransform()
{
    mBackgroundItem->updateTransform({ mLocation, mWidth, mHeight });
}