#pragma once

#include "FontItem.h"
#include "ITheme.h"
#include "WidgetState.h"
#include "BoxGeometry.h"

#include <memory>
#include <vector>
#include <string>

class MouseEvent;
class KeyboardEvent;
class PaintEvent;

class AbstractVisualNode;
class TransformNode;
class RectangleNode;

namespace mt
{
class Window;
}

class Widget : public BoxGeometry
{
public:
    Widget();

    virtual ~Widget();

    static std::unique_ptr<Widget> Create();

    virtual void addWidget(std::unique_ptr<Widget> widget);

    TransformNode* getRootNode() const;

    const std::string& getName() const;

    mt::Window* getTopLevelWindow() const;

    bool needsUpdate() const;

    virtual void onPaintEvent(const PaintEvent* event);

    virtual bool onMouseEvent(const MouseEvent* event);

    virtual bool onKeyboardEvent(const KeyboardEvent* event);

    void setBackground(ThemeToken::SystemToken token);

    void setVisible(bool visible);

    void setName(const std::string& name);

    void setWindow(mt::Window* window);

protected:
    virtual bool onMyKeyboardEvent(const KeyboardEvent* event);

    virtual void onMyMouseEvent(const MouseEvent* event);

    virtual void updateBackground(const PaintEvent* event);

    virtual void doPaint(const PaintEvent* event);

    virtual void updateChildLocations();

    virtual bool isDirty() const;

    void setParent(Widget* parent);

    Widget* getParent() const;

    mt::Window* getWindow() const;

    std::unique_ptr<TransformNode> mRootNode;
    std::vector<std::unique_ptr<Widget> > mChildren;

    unsigned mBorderThickness{0};
    ThemeToken::SystemToken mBackground;
    ThemeToken::SystemToken mBorder;

    bool mVisible{true};

    std::unique_ptr<RectangleNode> mBackgroundNode;

    bool mMaterialDirty{true};
    bool mVisibilityDirty{true};

    std::string mName;
    std::vector<TransformNode*> mPendingChildNodes;

    FontItem mDefaultFont;
    Widget* mParent{nullptr};
    mt::Window* mWindow{nullptr};
};

using WidgetUPtr = std::unique_ptr<Widget>;
